(* ::Package:: *)

BeginPackage["CV`"];
Needs["JLink`"];
InstallJava[];
AddToClassPath[$BaseDirectory<>"/Applications/CV/jmathcv.jar"];

CVOpen[]:=Module[{},
    Install[$BaseDirectory<>"/Applications/CV/opencvlink"];
    iCVOpen[];
]

CVClose[]:=Module[{},
    iCVClose[]; 
    Uninstall[$BaseDirectory<>"/Applications/CV/opencvlink"];
]

CVLoadImage[filename_,iscolor_]:=Module[{},
    If[FileExistsQ[filename],
        Return[iCVLoadImage[AbsoluteFileName[filename],iscolor]],
        Print["Error : file does not exist"]; Abort[];
        ];
]

CVCaptureFromFile[filename_]:=Module[{},
    If[FileExistsQ[filename],
        Return[iCVCaptureFromFile[AbsoluteFileName[filename]]],
        Print["Error : file does not exist"]; Abort[];
        ];
]

CVJOpenImageEditor[name_]:=Module[{frame},
    frame=JavaNew["jmathcv.ShowImage"];
    frame@setSize[400,400];
    frame@setTitle[name];
    frame@setModal[False];
    JavaShow[frame];
    Return[frame];
]

CVJImageEditor[name_,modal_]:=Module[{frame},
    frame=JavaNew["jmathcv.ShowImage"];
    frame@setSize[400,400];
    frame@setTitle[name];
    frame@setModal[modal];
    If[!modal,JavaShow[frame]];
    Return[frame];
    
]

CVJShowImage[frame_,img_]:=Module[{colorModel,size,data,image},
    size = CVGetSize[img];
    LoadJavaClass["java.awt.image.ColorModel"];
    colorModel = ColorModel`getRGBdefault[];
    data = CVGetIntImageArray[img];
    bitmap = JavaNew["java.awt.image.MemoryImageSource", size[[1]], size[[2]], colorModel, Flatten[data], 0, size[[1]]];
    image = frame@getToolkit[]@createImage[bitmap];
	frame@setImage[image];
    buttonListener = JavaNew["com.wolfram.jlink.MathActionListener"];
	buttonListener@setHandler["actionPerformed",ToString[returnFunction]];
	button = frame@mathkernelButton;
	(*button@addActionListener[buttonListener];*)
	returnFunction[_,_]:=Module[{},Print["ok"]];
	If[frame@isModal[],JavaShow[frame]];
]

CVJShowVideoFrames[frame_,video_,skip_]:=Module[{img,colorModel,size,data,image,count},
	img = CVGetFrame[video,1];
	count = 0;
	size = CVGetSize[img];
    LoadJavaClass["java.awt.image.ColorModel"];
    colorModel = ColorModel`getRGBdefault[];
    data = CVGetIntImageArray[img];
	CVReleaseImage[img];
    bitmap = JavaNew["java.awt.image.MemoryImageSource", size[[1]], size[[2]], colorModel, Flatten[data], 0, size[[1]]];
    image = frame@getToolkit[]@createImage[bitmap];
	frame@setImage[image];
    buttonListener = JavaNew["com.wolfram.jlink.MathActionListener"];
	buttonListener@setHandler["actionPerformed",ToString[returnFunction]];
	button = frame@mathkernelButton;
	button@setToolTipText["Image suivante"];
	button@addActionListener[buttonListener];
	returnFunction[_,_]:=Module[{},
		data = CVGetIntFrameArray[video,skip];
		count = count+skip;
		If[Length[data]>0,
		    bitmap = JavaNew["java.awt.image.MemoryImageSource", size[[1]], size[[2]], colorModel, Flatten[data], 0, size[[1]]];
			image = frame@getToolkit[]@createImage[bitmap];
			frame@setImage[image];
			frame@printMessage["Image "<>ToString[count]<>"\n"];,
			button@removeActionListener[buttonListener];
			frame@printMessage["Fin de la video\n"];
		]
	];
	If[frame@isModal[],JavaShow[frame]];
]

CVJGetVideoFramesPoints[frame_,video_,skip_]:=Module[{img,colorModel,size,data,image,count,pixelPointList,userPointList,ppl,upl},
    pixelPointList={};
    userPointList={};
	img = CVGetFrame[video,1];
	count = 0;
	size = CVGetSize[img];
    LoadJavaClass["java.awt.image.ColorModel"];
    colorModel = ColorModel`getRGBdefault[];
    data = CVGetIntImageArray[img];
	CVReleaseImage[img];
    bitmap = JavaNew["java.awt.image.MemoryImageSource", size[[1]], size[[2]], colorModel, Flatten[data], 0, size[[1]]];
    image = frame@getToolkit[]@createImage[bitmap];
	frame@setImage[image];
    buttonListener = JavaNew["com.wolfram.jlink.MathActionListener"];
	buttonListener@setHandler["actionPerformed",ToString[returnFunction]];
	button = frame@mathkernelButton;
	button@setToolTipText["Image suivante"];
	button@addActionListener[buttonListener];
	returnFunction[_,_]:=Module[{},
	    ppl=CVJGetPixelPointList[frame];
	    upl=CVJGetUserPointList[frame];
	    If[Length[ppl]>0,
	        frame@clearPointList[];
	        userPointList=Append[userPointList,upl];
	        pixelPointList=Append[pixelPointList,ppl];
	    ];
		data = CVGetIntFrameArray[video,skip];
		count = count+skip;
		If[Length[data]>0,
		    bitmap = JavaNew["java.awt.image.MemoryImageSource", size[[1]], size[[2]], colorModel, Flatten[data], 0, size[[1]]];
			image = frame@getToolkit[]@createImage[bitmap];
			frame@setImage[image];
			frame@printMessage["Image "<>ToString[count]<>"\n"];,
			button@removeActionListener[buttonListener];
			frame@printMessage["Fin de la video\n"];
		]
	];
	If[frame@isModal[],JavaShow[frame]];
	Return[{pixelPointList,userPointList}];
]

CVJGetPixelPointList[frame_]:=Module[{list,n,k,points},
    list = frame@getPixelPointList[];
    n = list@size[];
    points = {};
    For[k=0,k<n,k++,
        p = list@get[k];
        points = Append[points,{p@x,p@y}];
    ];
    Return[points];
]

CVJGetUserPointList[frame_]:=Module[{list,n,k,points},
    list = frame@getUserPointList[];
    n = list@size[];
    points = {};
    For[k=0,k<n,k++,
        p = list@get[k];
        points = Append[points,{p@x,p@y}];
    ];
    Return[points];
]

CVJGetPixelRectangleList[frame_]:=Module[{list,n,k,points},
    list = frame@getPixelRectangleList[];
    n = list@size[];
    points = {};
    For[k=0,k<n,k++,
        p = list@get[k];
        points = Append[points,{p@x,p@y,p@width,p@height}];
    ];
    Return[points];
]

CVJGetUserRectangleList[frame_]:=Module[{list,n,k,points},
    list = frame@getRectanglePointList[];
    n = list@size[];
    points = {};
    For[k=0,k<n,k++,
        p = list@get[k];
        points = Append[points,{p@x,p@y,p@width,p@height}];
    ];
    Return[points];
]

CVJGetPixel2UserMatrix[frame_]:=Module[{m},
    m = frame@getPixel2UserMatrix[];
    Return[m];
]

CVAffineTransform[m_,points_]:=Module[{n,p,liste,k,x,y},
    n=Length[points];
    liste={};
    For[k=1,k<=n,k++,
        p = points[[k]];
        x = m[[1]]*p[[1]]+m[[2]]*p[[2]]+m[[5]];
        y = m[[3]]*p[[1]]+m[[4]]*p[[2]]+m[[6]];
        liste = Append[liste,{x,y}];
    ];
    Return[liste];
]

CVFlipY[size_,points_]:=Table[{points[[k,1]],size[[2]]-points[[k,2]]},{k,1,Length[points]}]

EndPackage[];
