/*---------------------------------------------------------------------
Copyright Frédéric Legrand (2010)

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

-------------------------------------------------------------------------*/
#include <stdlib.h>
#include <stdio.h>
#include "mathlink.h"
#include "cv.h"
#include "highgui.h"

#define RETURN MLPutString(stdlink,""); return;

/*------------------------------- OpenCV structures references ----------------------------------------------------------*/
#define DEFAULT_TABREF_SIZE 10000
#define EMPTY_REF 0
#define IPLIMAGE_REF 1
#define CVCAPTURE_REF 2
#define CVMAT_REF 3
#define CVSEQ_REF 4
#define CVHISTOGRAM_REF 5
#define CVMEMSTORAGE_REF 6
#define CVVIDEOWRITER_REF 7

#define GETPOINTER(name,ref,type,reftype) name = (type) _get_pointer(ref,reftype); if (!name) {MLPutString(stdlink,"Error : bad reference"); return;}
#define GETCVARRPOINTER(name,ref) name = (CvArr*) _get_cvarr_pointer(ref); if (!name) {MLPutString(stdlink,"Error : bad reference"); return;}

typedef struct {
    void *pointer;
    int type;
} reference;

typedef struct {
    IplImage *image;
    CvSeq *points;
    char *window;
    int *color;
} imagePoints;


reference **tabRef;
int tabRefSize = DEFAULT_TABREF_SIZE;
int lastRef = 0;
CvPoint lastMousePoint;
int mouseEventDone;

int _append_ref(void *pointer, int type) {
    if (pointer==NULL) return 0;
    tabRef[lastRef] = (reference*) malloc(sizeof(reference));
    tabRef[lastRef]->pointer = pointer;
    tabRef[lastRef]->type = type;
    lastRef += 1;
    if (lastRef == tabRefSize) {
        tabRefSize += DEFAULT_TABREF_SIZE;
        tabRef = realloc(tabRef,tabRefSize*sizeof(reference*));
    }
    return lastRef-1;
}

void cv_open() {
    tabRef = malloc(tabRefSize*sizeof(reference*));
    tabRef[0] = (reference*) malloc(sizeof(reference));
    tabRef[0]->pointer = NULL;
    tabRef[0]->type = EMPTY_REF;
    lastRef = 1;
    lastMousePoint.x = 0; lastMousePoint.y = 0; 
    MLPutString(stdlink,"CV opened");
}

void* _get_pointer(int ref, int type) {
    if ((ref<=0)||(ref>=lastRef)) return NULL;
    if (tabRef[ref]->type != type) return NULL;
    return tabRef[ref]->pointer;
}

void* _get_cvarr_pointer(int ref) {
    if ((ref<=0)||(ref>=lastRef)) return NULL;
    if ((tabRef[ref]->type != IPLIMAGE_REF)&&(tabRef[ref]->type != CVMAT_REF)&&(tabRef[ref]->type != CVSEQ_REF)) return NULL;
    return tabRef[ref]->pointer;
}
/* ne fonctionne pas avec gcc
void cv_close() {
    int k;
    for (k=0; k < lastRef; k++) {
        switch (tabRef[k]->type) {
            case IPLIMAGE_REF:
                cvReleaseImage(&(IplImage*)(tabRef[k]->pointer));
                break;
            case CVCAPTURE_REF:
                cvReleaseCapture(&(CvCapture*)(tabRef[k]->pointer));
                break;
            case CVMAT_REF:
                cvReleaseMat(&(CvMat*)(tabRef[k]->pointer));
                break;
            case CVHISTOGRAM_REF:
                cvReleaseHist(&(CvHistogram*)(tabRef[k]->pointer));
                break;
            case CVMEMSTORAGE_REF:
                cvReleaseMemStorage(&(CvMemStorage*)(tabRef[k]->pointer));
            case CVVIDEOWRITER_REF:
                cvReleaseVideoWriter(&(CvVideoWriter*)(tabRef[k]->pointer));
        }
        free(tabRef[k]);
    }
    MLPutString(stdlink,"CV closed");
}*/

void cv_close() {
    int k;
    for (k=0; k < lastRef; k++) {
        switch (tabRef[k]->type) {
            case IPLIMAGE_REF:
                cvReleaseImage((IplImage**)&(tabRef[k]->pointer));
                break;
            case CVCAPTURE_REF:
                cvReleaseCapture((CvCapture**)&(tabRef[k]->pointer));
                break;
            case CVMAT_REF:
                cvReleaseMat((CvMat**)&(tabRef[k]->pointer));
                break;
            case CVHISTOGRAM_REF:
                cvReleaseHist((CvHistogram**)&(tabRef[k]->pointer));
                break;
            case CVMEMSTORAGE_REF:
                cvReleaseMemStorage((CvMemStorage**)&(tabRef[k]->pointer));
            case CVVIDEOWRITER_REF:
                cvReleaseVideoWriter((CvVideoWriter**)&(tabRef[k]->pointer));
        }
        free(tabRef[k]);
    }
    MLPutString(stdlink,"CV closed");
}

//--------------------------------------  OPERATIONS ON ARRAYS ----------------------------



void cv_convert_scale(int srcRef, int dstRef, double scale, double shift) {
    CvArr *src, *dst;
    GETCVARRPOINTER(src,srcRef)
    GETCVARRPOINTER(dst,dstRef)
    cvConvertScale(src,dst,scale,shift);
    RETURN
}

void cv_copy(int srcRef, int dstRef, int maskRef) {
    CvArr *src, *dst, *mask;
    GETCVARRPOINTER(src,srcRef)
    GETCVARRPOINTER(dst,dstRef)
    mask = (CvArr*)_get_cvarr_pointer(maskRef);
    cvCopy(src,dst,mask);
    RETURN
}

void cv_create_image(int *size, int sizelen, int depth, int channels) {
    
    if (sizelen==2) {
        MLPutInteger(stdlink,_append_ref((void*) cvCreateImage(cvSize(size[0],size[1]),depth,channels),IPLIMAGE_REF));
    }
    else {
        MLPutString(stdlink,"Error : size must be a list of two integers");
    }
}

void cv_create_mat(int rows, int cols, int depthtype, int channels) {
    MLPutInteger(stdlink,_append_ref((void*) cvCreateMat(rows,cols,CV_MAKETYPE(depthtype,channels)),CVMAT_REF));
}

void cv_create_mat_header(int rows, int cols, int depthtype, int channels) {
    MLPutInteger(stdlink,_append_ref((void*) cvCreateMat(rows,cols,CV_MAKETYPE(depthtype,channels)),CVMAT_REF));
}



void cv_get_row(int refarr, int refsubmat, int row) {
    CvArr *arr;
    CvMat *submat;
    GETCVARRPOINTER(arr,refarr)
    GETPOINTER(submat,refsubmat,CvMat*,CVMAT_REF)
    cvGetRow(arr,submat,row);
    RETURN
}

void cv_get_col(int refarr, int refsubmat, int col) {
    CvArr *arr;
    CvMat *submat;
    GETCVARRPOINTER(arr,refarr)
    GETPOINTER(submat,refsubmat,CvMat*,CVMAT_REF)
    cvGetCol(arr,submat,col);
    RETURN
}

void cv_get_size(int ref) {
    CvArr *arr;
    CvSize size;
    int s[2];
    if (arr=_get_cvarr_pointer(ref)) {
        size = cvGetSize(arr);
        s[0] = size.width; s[1] = size.height;
        MLPutIntegerList(stdlink,s,2);
    }
    else {
        MLPutString(stdlink,"Error : bad CvArr reference");
    }
}

void cv_get_sub_rect(int arrRef,int submatRef,int *rect, int rectlen) {
    CvArr *arr, *submat;
    GETCVARRPOINTER(arr,arrRef)
    GETPOINTER(submat,submatRef,CvMat*,CVMAT_REF)
    if (rectlen!=4) {
        MLPutString(stdlink,"Error : arg 3 must be a list of 4 integers");
        return;
    }
    cvGetSubRect(arr,submat,cvRect(rect[0],rect[1],rect[2],rect[3]));
    RETURN
}

void cv_min_max_loc(int arrRef) {
    CvArr* arr;
    double min_val, max_val;
    CvPoint min_loc, max_loc;
    double a[6];
    int dims[2];
    GETCVARRPOINTER(arr,arrRef)
    cvMinMaxLoc(arr,&min_val,&max_val,&min_loc,&max_loc,NULL);
    dims[0] = 3; dims[1] = 2;
    a[0] = min_val; a[1] = max_val; a[2] = min_loc.x; a[3] = min_loc.y; a[4] = max_loc.x; a[5] = max_loc.y;
    MLPutReal64Array(stdlink,a,dims,NULL,2);
}

void cv_release_image(int ref) {
    IplImage *image;
    if (image=(IplImage*) _get_pointer(ref,IPLIMAGE_REF)) {
        cvReleaseImage(&image);
        tabRef[ref]->type = EMPTY_REF;
        MLPutString(stdlink,"");
    }
    else {
        MLPutString(stdlink,"Error : bad IplImage reference");
    }
}

void cv_reset_image_roi(int imgRef) {
    IplImage *img;
    GETPOINTER(img,imgRef,IplImage*,IPLIMAGE_REF)
    cvResetImageROI(img);
    RETURN
}

void cv_set_image_roi(int imgRef, int *rect, int rectlen) {
    IplImage *img;
    GETPOINTER(img,imgRef,IplImage*,IPLIMAGE_REF)
    if (rectlen!=4) {
        MLPutString(stdlink,"Error : arg 2 must be a list of 4 integers");
        return;
    }
    cvSetImageROI(img,cvRect(rect[0],rect[1],rect[2],rect[3]));
    RETURN
}


void cv_split(int refsrc, int refdst0, int refdst1, int refdst2, int refdst3) {
    CvArr *src, *dst0, *dst1, *dst2, *dst3;
    GETCVARRPOINTER(src,refsrc)
    GETCVARRPOINTER(dst0,refdst0)
    GETCVARRPOINTER(dst1,refdst1)
    GETCVARRPOINTER(dst2,refdst2)
    dst3 = (CvArr*) _get_cvarr_pointer(refdst3);
    cvSplit(src,dst0,dst1,dst2,dst3);
    MLPutString(stdlink,"");
}
//------------------------------ extra operations on arrays ------------------------------------------------

void cv_xy_mean(int imgRef, int *rect, int rectlen) {
    IplImage *img;
    CvRect rectangle;
    CvMat *submat;
    double xmean,ymean;
    int npixels;
    int row,col;
    unsigned char *ptr;
    double xy[2];
    GETPOINTER(img,imgRef,IplImage*,IPLIMAGE_REF)
    if (rectlen!=4) {
        MLPutString(stdlink,"Error : arg 2 must be a list of 4 integers");
        return;
    }
    if ((img->nChannels!=1)||(img->depth!=8)) {
        MLPutString(stdlink,"Error : image must be a 1 channel 8 bits image");
        return;
    }
    rectangle = cvRect(rect[0],rect[1],rect[2],rect[3]);
    if (rectangle.x < 0) rectangle.x = 0;
    if (rectangle.y < 0) rectangle.y = 0;
    if (rectangle.x+rectangle.width > img->width) rectangle.width = img->width - rectangle.x ;
    if (rectangle.y+rectangle.height > img->height) rectangle.height = img->height - rectangle.y ;
    if (rectangle.width < 0) rectangle.width = 0;
    if (rectangle.height < 0) rectangle.height = 0;
    submat = cvCreateMat(rectangle.width,rectangle.height,CV_8UC1);
    cvGetSubRect(img,submat,rectangle);
    xmean = 0; ymean = 0; npixels = 0;
    for (row=0; row<submat->rows; row++) {
           ptr = submat->data.ptr+row*submat->step;
           for (col=0; col<submat->cols; col++) {
               if (*ptr) {
                    xmean += col+rectangle.x;
                    ymean += row+rectangle.y;
                    npixels += 1;
               }
               ptr += 1;
           }
    }
    xmean = xmean / npixels;
    ymean = ymean / npixels;
    xy[0] = xmean; xy[1] = ymean;
    MLPutReal64List(stdlink,xy,2);
}

//--------------------------------- exchange array data width mathematica kernel------------------------------------
void _get_2darray(CvArr* arr, int channel) {
    int rows, cols;
    double *array;
    long dims[2];
    int i,j,k;
    CvScalar scalar;
    rows = cvGetDimSize(arr,0);
    cols = cvGetDimSize(arr,1);
    array = malloc(sizeof(double)*rows*cols);
    dims[0] = rows;
    dims[1] = cols;
    k = 0;
    for (i=0; i<cols; i++) 
            for (j=0; j<rows; j++) {
                scalar = cvGet2D(arr,j,i);
                array[k] = scalar.val[channel];
                k += 1;
    }
    MLPutRealArray(stdlink,array,dims,NULL,2);
}

void cv_get_image_array(int imgRef, int norm, int order) {
	IplImage *img;
	double *array;
	long dims[3];
	long d;
	int i,j,c,k;
	CvScalar scalar;
	double scale = 1.0;
	GETPOINTER(img,imgRef,IplImage*,IPLIMAGE_REF)
	dims[0] = img->height;
	dims[1] = img->width;
	dims[2] = img->nChannels;
	if (dims[2]==1) d=2; else d=3;
	array = malloc(sizeof(double)*dims[0]*dims[1]*dims[2]);
	if (norm==1) {
	    switch (img->depth) {
	        case IPL_DEPTH_8U: scale = 0xFF; break;
	        case IPL_DEPTH_16U: scale = 0xFFFF; break;
	    }
	}
	k = 0;
	if (order==1) {
	for (i=0; i<dims[0]; i++)
		for (j=0; j<dims[1]; j++) {
			scalar = cvGet2D(img,i,j);
			for (c=0; c<dims[2]; c++) {
				array[k] = scalar.val[c]/scale;
				k += 1;
			}
		}
	}
	else {
	  for (i=0; i<dims[0]; i++)
		for (j=0; j<dims[1]; j++) {
			scalar = cvGet2D(img,i,j);
			for (c=dims[2]-1; c>=0; c--) {
				array[k] = scalar.val[c]/scale;
				k += 1;
			}
		}  
	}
	MLPutRealArray(stdlink,array,dims,NULL,d);
	free(array);
}

void _get_int_image_array(IplImage *img) {
    unsigned int *array;
    long dims[2];
    int x,y,k;
    int ch;
    uchar* ptr;
    short* ptr1;
    float b,r,g;
    unsigned char ic2,ic3;
    dims[0] = img->height;
	dims[1] = img->width;
	array = malloc(sizeof(unsigned int)*dims[0]*dims[1]);
	ch = img->nChannels;
	ic2=1; ic3=2;
	if (ch==1) {ic2=0; ic3=0;}
	else if (ch==2) {ic2=1; ic3=1;}
	k = 0;
	if (img->depth==IPL_DEPTH_8U) {
	for (y=0; y < img->height; y++) {
	    ptr = (uchar*)(img->imageData+y*img->widthStep);
	    for (x=0; x< img->width; x++) {
	         array[k] = ptr[ch*x] | (ptr[ch*x+ic2] << 8) | (ptr[ch*x+ic3] << 16) | 0xFF000000;
	         k += 1;
	    }
	}}
	else if (img->depth==IPL_DEPTH_16U) {
	    for (y=0; y < img->height; y++) {
	        ptr1 = (short*)(img->imageData+y*img->widthStep);
	        for (x=0; x< img->width; x++) {
	            b = ((float)ptr1[ch*x])/256.0f;
	            r = ((float)ptr1[ch*x+ic2])/256.0f;
	            g = ((float)ptr1[ch*x+ic3])/256.0f;
	            array[k] = ((unsigned char)b) | (((unsigned char)g) << 8) | (((unsigned char)r) << 16) | 0xFF000000;
	            k += 1;
	        }
	    }
	}
	MLPutIntegerArray(stdlink,array,dims,NULL,2);
	free(array);
}

void cv_get_int_image_array(int imgRef) {
    IplImage *img;
    GETPOINTER(img,imgRef,IplImage*,IPLIMAGE_REF)
	_get_int_image_array(img);
}

void cv_get_int_frame_array(int captRef, int skip) {
    CvCapture *capture;
    IplImage *frame = 0;
    int f;
    GETPOINTER(capture,captRef,CvCapture*,CVCAPTURE_REF)
    if (skip <= 0) {
        MLPutString(stdlink,"Error : skip must be > 0");
        return;
    }
    for (f=0; f < skip-1; f++)  cvGrabFrame(capture);
    frame = cvQueryFrame(capture);
    if (frame) {
        _get_int_image_array(frame);
    }
    else {
        MLPutInteger(stdlink,0);   
    }
}

void _get_array_loop(CvArr *cvarr, double *array, long *dims, int *idx, int level, long d, int *k, int channel) {
    CvScalar scalar;
    for (idx[level]=0; idx[level]<dims[level]; idx[level]++) {
        if (level < d-1) _get_array_loop(cvarr,array,dims,idx,level+1,d,k,channel);
        else {
            scalar = cvGetND(cvarr,idx);
            array[*k] = scalar.val[channel];
            *k += 1;
        }
    }
}

void _set_array_loop(CvArr *cvarr, double *array, long *dims, int *idx, int level, long d, int *k, int channel) {
    CvScalar scalar;
    for (idx[level]=0; idx[level]<dims[level]; idx[level]++) {
        if (level < d-1) _set_array_loop(cvarr,array,dims,idx,level+1,d,k,channel);
        else {
            scalar = cvGetND(cvarr,idx);
            scalar.val[channel] = array[*k];
            cvSetND(cvarr,idx,scalar);
            *k += 1;
        }
    }
}

void _get_array(CvArr* arr, int channel) {
    long *dims;
    long d;
    int k;
    double *array;
    int *idx;
    int nb;
    d = cvGetDims(arr,NULL);
    dims = malloc(sizeof(int)*d);
    idx = malloc(sizeof(int)*d);
    nb = 1;
    for (k=0; k<d; k++) {
        dims[k] = cvGetDimSize(arr,k);
        nb *= dims[k];
    }
    array = malloc(sizeof(double)*nb);
    k = 0;
    _get_array_loop(arr,array,dims,idx,0,d,&k,channel);
    MLPutRealArray(stdlink,array,dims,NULL,d);
    free(dims);
    free(idx);
    free(array);
}

void cv_get_array(int refarr, int channel) {
    CvArr* arr;
    arr = (CvArr*)_get_cvarr_pointer(refarr);
    if (arr==NULL) {
        MLPutString(stdlink,"Error : bad CvArr reference");
        return;
    }
    if ((channel<1)||(channel>4)) {
        MLPutString(stdlink,"Error : bad channel number");
        return;
    }
    _get_array(arr,channel-1);
    return;
    
}

void cv_set_array(int arrRef, int channel) {
    CvArr* arr;
    double *data;
    long *dims;
    char **heads;
    long d;
    int k;
    int *idx;
    GETCVARRPOINTER(arr,arrRef)
    if ((channel<1)||(channel>4)) {
        MLPutString(stdlink,"Error : bad channel number");
        return;
    }
    if (!MLGetRealArray(stdlink,&data,&dims,&heads,&d)) {
        MLPutString(stdlink,"Error : arg 3 must be an array");
        return;
    }
    if (d!=cvGetDims(arr,NULL)) {
        MLPutString(stdlink,"Error : bad array dimension");
        MLDisownRealArray(stdlink,data,dims,heads,d);
        return;
    }
    for (k=0; k<d; k++) {
        if (dims[k]!=cvGetDimSize(arr,k)) {
               MLPutString(stdlink,"Error : bad array dimension");
               MLDisownRealArray(stdlink,data,dims,heads,d);
               return;
        }
    }
    idx = malloc(sizeof(int)*d);
    k=0;
    _set_array_loop(arr,data,dims,idx,0,d,&k,channel-1);
    free(idx);
    MLDisownRealArray(stdlink,data,dims,heads,d);
    RETURN
}

void cv_get_hist_array(int histRef) {
    CvHistogram *hist;
    hist = (CvHistogram*) _get_pointer(histRef,CVHISTOGRAM_REF);
    if (!hist) {
        MLPutString(stdlink,"Error : bad histogram reference");
        return;
    }
    _get_array(hist->bins,0);
}

void cv_image_info(int ref) {
    IplImage *image;
    char reponse[200];
    char depth[4];
    if (image=(IplImage*) _get_pointer(ref,IPLIMAGE_REF)) {
        switch (image->depth) {
            case (IPL_DEPTH_8U):
#ifdef WIN32
                sprintf_s(depth,3,"8U");
#else
                sprintf(depth,"8U");
#endif
                break;
            case (IPL_DEPTH_16U):
#ifdef WIN32
                sprintf_s(depth,3,"16U");
#else
                sprintf(depth,"16U");
#endif
                break;
        }
#ifdef WIN32
        sprintf_s(reponse,200,"nChannels : %d\ndepth : %s\nwidth : %d\nheight : %d",image->nChannels,depth,image->width,image->height);
#else
        sprintf(reponse,"nChannels : %d\ndepth : %s\nwidth : %d\nheight : %d",image->nChannels,depth,image->width,image->height);
#endif
        MLPutString(stdlink,reponse);
    }
    else {
        MLPutString(stdlink,"Error: bad IplImage reference");
    }
}



//------------------------------------- DYNAMIC STRUCTURES --------------------------------------
void cv_create_mem_storage(int blocksize) {
    MLPutInteger(stdlink,_append_ref((void*)cvCreateMemStorage(blocksize),CVMEMSTORAGE_REF));
    
}

void cv_get_seq_hnext(int seqRef) {
    CvSeq *seq;
    GETPOINTER(seq,seqRef,CvSeq*,CVSEQ_REF)
    MLPutInteger(stdlink,_append_ref((void*) (seq->h_next),CVSEQ_REF));
}

void cv_get_seq_hprev(int seqRef) {
    CvSeq *seq;
    GETPOINTER(seq,seqRef,CvSeq*,CVSEQ_REF)
    MLPutInteger(stdlink,_append_ref((void*) (seq->h_prev),CVSEQ_REF));
}

void cv_get_seq_vnext(int seqRef) {
    CvSeq *seq;
    GETPOINTER(seq,seqRef,CvSeq*,CVSEQ_REF)
    MLPutInteger(stdlink,_append_ref((void*) (seq->v_next),CVSEQ_REF));
}

void cv_get_seq_vprev(int seqRef) {
    CvSeq *seq;
    GETPOINTER(seq,seqRef,CvSeq*,CVSEQ_REF)
    MLPutInteger(stdlink,_append_ref((void*) (seq->v_prev),CVSEQ_REF));
}

void cv_get_seq_size(int seqRef) {
    CvSeq *seq;
    GETPOINTER(seq,seqRef,CvSeq*,CVSEQ_REF)
    if (seq) MLPutInteger(stdlink,seq->total);
    else MLPutInteger(stdlink,0);
}

//---------------------------------DRAWING FUNCTIONS---------------------------
void cv_draw_contours(int imgRef ,int contoursRef, int *externalColor, int externalColorLen, int *holeColor, int holeColorLen, int maxLevel, int thickness, int lineType, int *offset, int offsetLen) {
    CvArr *img;
    CvSeq *contours;
    img = (CvArr*)_get_cvarr_pointer(imgRef);
    if (!img) {
        MLPutString(stdlink,"Error : bad CvArr reference");
        return;
    }
    contours = (CvSeq*) _get_pointer(contoursRef,CVSEQ_REF);
    if (!contours) {
        MLPutString(stdlink,"Error : bad CvSeq reference");
        return;
    }
    if ((externalColorLen!=3)||(holeColorLen!=3)) {
        MLPutString(stdlink,"Error : color must be a list of 3 integers");
        return;
    }
    if (offsetLen!=2) {
        MLPutString(stdlink,"Error : offset must be a list of 2 integers");
        return;
    }
    cvDrawContours(img,contours,CV_RGB(externalColor[0],externalColor[1],externalColor[2]),CV_RGB(holeColor[0],holeColor[1],holeColor[2]),maxLevel,thickness,lineType,cvPoint(offset[0],offset[1]));
    MLPutString(stdlink,"");
}
void cv_rectangle(int imgRef, int *rect, int rectlen, int *color, int colorlen, int thickness) {
    CvArr *img;
    GETCVARRPOINTER(img,imgRef)
    if (rectlen!=4) {
        MLPutString(stdlink,"Error : arg 2 must be a list of 4 integers");
        return;
    }
    if (colorlen!=3) {
        MLPutString(stdlink,"Error : arg 3 must be a list of 3 integers");
        return;
    }
    cvRectangle(img,cvPoint(rect[0],rect[1]),cvPoint(rect[0]+rect[2],rect[1]+rect[3]),CV_RGB(color[0],color[1],color[2]),thickness,8,0);
    RETURN
}
/*------------------------------------------------ Image filtering ----------------------------------------------------*/
void cv_filter_2d(int srcref, int dstref, int kernelref) { 
    CvArr *src, *dst;
    CvMat* kernel;
    GETCVARRPOINTER(src,srcref)
    GETCVARRPOINTER(dst,dstref)
    GETPOINTER(kernel,kernelref,CvMat*,CVMAT_REF)
    cvFilter2D(src,dst,kernel,cvPoint(-1,-1));
    RETURN
}
void cv_laplace(int srcref, int dstref, int aperture) {
    CvArr *src, *dst;
    GETCVARRPOINTER(src,srcref)
    GETCVARRPOINTER(dst,dstref)
    cvLaplace(src,dst,aperture);
    RETURN    
}
void cv_sobel(int srcref, int dstref, int xorder, int yorder, int aperture) {
    CvArr *src, *dst;
    GETCVARRPOINTER(src,srcref)
    GETCVARRPOINTER(dst,dstref)
    cvSobel(src,dst,xorder,yorder,aperture);
    RETURN
}
void cv_smooth(int srcref, int dstref, int smoothtype, int param1, int param2, double param3) {
    CvArr *src, *dst;
    GETCVARRPOINTER(src,srcref)
    GETCVARRPOINTER(dst,dstref)
    cvSmooth(src,dst,smoothtype,param1,param2,param3,0);
    RETURN
}
/*-------------------------------------------------Miscellaneous Image Transformations ---------------------------------*/
void cv_cvt_color(int srcRef, int dstRef, int code) {
    CvArr *src, *dst;
    GETCVARRPOINTER(src,srcRef)
    GETCVARRPOINTER(dst,dstRef)
    cvCvtColor(src,dst,code);
    RETURN
}

void cv_threshold(int srcRef, int dstRef, double threshold, double  max_value, int threshold_type ) {
    CvArr *src, *dst;
    GETCVARRPOINTER(src,srcRef)
    GETCVARRPOINTER(dst,dstRef)
    cvThreshold(src,dst,threshold,max_value,threshold_type);
    RETURN
}
//---------------------------------HISTOGRAM ----------------------------------

void cv_calc_back_project(int *imgRefList, long imgRefListLength, int backProjectRef, int histRef) {
    IplImage **imgList;
    IplImage *img;
    CvArr *backProject;
    CvHistogram *hist;
    int k;
    imgList = malloc(sizeof(IplImage*)*imgRefListLength);
    for (k=0; k<imgRefListLength; k++) {
        img = (IplImage*)_get_pointer(imgRefList[k],IPLIMAGE_REF);
        if (!img) {
            MLPutString(stdlink,"Error : bad image reference");
            free(imgList);
            return;
        }
        imgList[k] = img;
    }
    GETCVARRPOINTER(backProject,backProjectRef)
    GETPOINTER(hist,histRef,CvHistogram*,CVHISTOGRAM_REF)
    cvCalcBackProject(imgList,backProject,hist);
    free(imgList);
    RETURN
}

void cv_calc_hist(int *imgRefList, long imgRefListLength, int histRef, int accumulate, int arrRef) {
    IplImage **imgList;
    IplImage *img;
    CvHistogram *hist;
    CvArr *arr;
    int k;
    imgList = malloc(sizeof(IplImage*)*imgRefListLength);
    for (k=0; k<imgRefListLength; k++) {
        img = (IplImage*)_get_pointer(imgRefList[k],IPLIMAGE_REF);
        if (!img) {
            MLPutString(stdlink,"Error : bad image reference");
            free(imgList);
            return;
        }
        imgList[k] = img;
    }
    hist = (CvHistogram*)_get_pointer(histRef,CVHISTOGRAM_REF);
    if (!hist) {
        MLPutString(stdlink,"Error : bad histogram reference");
        free(imgList);
        return;
    }
    arr = (CvArr*)_get_cvarr_pointer(arrRef);
    cvCalcHist(imgList,hist,accumulate,arr);
    free(imgList);
    MLPutString(stdlink,"");
}

void cv_create_hist(int *sizes, int dimsizes, int type) {
    long *rangesdims;
    double *ranges_double;
    float **ranges;
    char **heads;
    long d;
    int uniform;
    int i,k;
    //char message[100];
    if (!MLGetRealArray(stdlink,&ranges_double,&rangesdims,&heads,&d)) {
        MLPutString(stdlink,"Error : bar range array");
        return;
    }
    if (rangesdims[0]!=dimsizes) {
        MLPutString(stdlink,"Error : incompatible dimensions");
        MLDisownRealArray(stdlink,ranges_double,rangesdims,heads,d);
        return;
    }
    MLGetInteger(stdlink,&uniform);
    ranges = malloc(sizeof(float*)*dimsizes);
    if (uniform) {
        if (rangesdims[1]!=2) {
            MLPutString(stdlink,"Error : incompatible dimensions");
            MLDisownRealArray(stdlink,ranges_double,rangesdims,heads,d);
            return;
        }
        k = 0;
        for (i=0; i < dimsizes; i++) {
            ranges[i] = malloc(sizeof(float)*2);
            ranges[i][0] = (float) ranges_double[k];
            k += 1;
            ranges[i][1] = (float) ranges_double[k];
            k += 1;
        }
    }
    else {
        ranges = NULL;
        // à faire
    }
    MLPutInteger(stdlink,_append_ref((void*)cvCreateHist(dimsizes,sizes,type,ranges,uniform),CVHISTOGRAM_REF));
    //sprintf_s(message,100,"ranges : %f,%f",ranges[0][0],ranges[0][1]);
    //MLPutString(stdlink,message);
    MLDisownRealArray(stdlink,ranges_double,rangesdims,heads,d);
    for (i=0; i < dimsizes; i++) free(ranges[i]);
    free(ranges);
}

void cv_get_minmax_hist_value(int histref) {
    CvHistogram *hist;
    int *min_idx, *max_idx;
    float min_value, max_value;
    double *array;
    int d;
    int dims[2];
    int i;
    GETPOINTER(hist,histref,CvHistogram*,CVHISTOGRAM_REF)
    d = hist->mat.dims;
    min_idx = malloc(sizeof(int)*d);
    max_idx = malloc(sizeof(int)*d);
    dims[0] = 2;
    dims[1] = 1+d;
    cvGetMinMaxHistValue(hist,&min_value,&max_value,min_idx,max_idx);
    array = malloc(sizeof(double)*dims[0]*dims[1]);
    array[0] = min_value; array[dims[1]] = max_value;
    for (i=0; i<d; i++) {
        array[1+i] = min_idx[i];
        array[1+i+dims[1]] = max_idx[i];
    }
    MLPutReal64Array(stdlink,array,dims,NULL,2);
    free(min_idx); free(max_idx);
}

void cv_normalize_hist(int histRef, double factor) {
       CvHistogram *hist;
       GETPOINTER(hist,histRef,CvHistogram*,CVHISTOGRAM_REF)
       cvNormalizeHist(hist,factor);
       RETURN
}

void cv_thresh_hist(int histRef, double threshold) {
       CvHistogram *hist;
       GETPOINTER(hist,histRef,CvHistogram*,CVHISTOGRAM_REF)
       cvThreshHist(hist,threshold);
       RETURN
}
//------------------- extra histogram functions -----------------------------------

void cv_convert_hist_scale(int histRef,double scale, double shift) {
    CvHistogram *hist;
    GETPOINTER(hist,histRef,CvHistogram*,CVHISTOGRAM_REF)
    cvConvertScale(hist->bins,hist->bins,scale,shift);
    RETURN
}

void cv_get_hue_sat_hist_image(int histRef, int size) {
    CvHistogram *hist;
    int nhue, nsat;
    int width, height;
    int h,s;
    float v;
    float hueRange,satRange;
    float min,max;
    IplImage *img;
    GETPOINTER(hist,histRef,CvHistogram*,CVHISTOGRAM_REF)
    nhue = hist->mat.dim[0].size;
    nsat = hist->mat.dim[1].size;
    width = nhue*size;
    height = nsat*size;
    img = cvCreateImage(cvSize(width,height),8,3);
    hueRange = 180.0; satRange = 255.0;
    cvGetMinMaxHistValue(hist,&min,&max,NULL,NULL);
    for (h=0; h<nhue; h++)
        for (s=0; s<nsat; s++) {
           v = (float) cvQueryHistValue_2D(hist,h,s);
           cvRectangle(img,cvPoint(h*size,(nsat-1-s)*size),cvPoint((h+1)*size,(nsat-s)*size),cvScalar(((float)h)/((float)nhue)*hueRange,((float)s)/((float)nsat)*satRange,v/max*255,0),CV_FILLED,8,0);
        }
    cvCvtColor(img,img,CV_HSV2BGR);
    MLPutInteger(stdlink,_append_ref((void*)img,IPLIMAGE_REF));
}
/*--------------------------------- Feature detection -------------------------------------------------*/
void cv_canny(int imgRef, int edgesRef, double threshold1, double threshold2, int aperture) {
    CvArr *img, *edges;
    GETCVARRPOINTER(img,imgRef)
    GETCVARRPOINTER(edges,edgesRef)
    cvCanny(img,edges,threshold1,threshold2,aperture);
    RETURN
}
void cv_hough_lines2(int imgRef, int memRef, int method, double rho, double theta, int threshold, double param1, double param2) {
    CvArr *image;
    CvMemStorage *mem;
    CvSeq *lines;
    GETCVARRPOINTER(image,imgRef)
    GETPOINTER(mem,memRef,CvMemStorage*,CVMEMSTORAGE_REF)
    lines=cvHoughLines2(image,mem,method,rho,theta,threshold,param1,param2);
    MLPutInteger(stdlink,_append_ref((void*)lines,CVSEQ_REF));
}
//--------------------------------- Feature detection : extra functions --------------------------------
void cv_get_hough_lines(int seqRef, int method) {
    CvSeq *seq;
    int k,i;
    float *data;
    int dim[3];
    float *line;
    CvPoint *points;
    GETPOINTER(seq,seqRef,CvSeq*,CVSEQ_REF)
    if ((method==CV_HOUGH_STANDARD)||(method==CV_HOUGH_MULTI_SCALE)) {
        dim[0] = seq->total;
        dim[1] = 2;
        data = malloc(sizeof(float)*dim[0]*dim[1]);
        for (k=0; k<seq->total; k++) {
           line = (float*) cvGetSeqElem(seq,k);
           data[2*k] = line[0]; 
           data[2*k+1] = line[1];
        }
        MLPutReal32Array(stdlink,data,dim,NULL,2);
    }
    else if (method==CV_HOUGH_PROBABILISTIC) {
        dim[0] = seq->total;
        dim[1] = 2;
        dim[2] = 2;
        data = malloc(sizeof(float)*dim[0]*dim[1]*dim[2]);
        i=0;
        for (k=0; k<seq->total; k++) {
            points = (CvPoint*) cvGetSeqElem(seq,k);
            data[i++] = (float)points[0].x;
            data[i++] = (float)points[0].y;
            data[i++] = (float)points[1].x;
            data[i++] = (float)points[1].y;
        }
        MLPutReal32Array(stdlink,data,dim,NULL,3);
    }
    free(data);
}
void cv_draw_hough_lines(int arrRef, int seqRef, int method, int *rgb, int rgbsize, int length, int thickness){
    CvArr *arr;
    CvSeq *seq;
    float *line;
    int k;
    CvScalar color;
    float rho0, theta0;
    double a,b,xk,yk;
    CvPoint pt1,pt2;
    CvPoint *points;
    GETCVARRPOINTER(arr,arrRef)
    GETPOINTER(seq,seqRef,CvSeq*,CVSEQ_REF)
    if (rgbsize!=3) {
        MLPutString(stdlink,"Error : rgb color must be a list of 3 integers");
        return;
    }
    color = CV_RGB(rgb[0],rgb[1],rgb[2]);
    if ((method==CV_HOUGH_STANDARD)||(method==CV_HOUGH_MULTI_SCALE)) {
        for (k=0; k<seq->total; k++) {
            line = (float*) cvGetSeqElem(seq,k);
            rho0 = line[0];
            theta0 = line[1];
            a = cos(theta0); b=sin(theta0);
            xk = a*rho0; yk = b*rho0;
            pt1.x = cvRound(xk-length*b); pt1.y = cvRound(yk+length*a);
            pt2.x = cvRound(xk+length*b); pt2.y = cvRound(yk-length*a);
            cvLine(arr,pt1,pt2,color,thickness,8,0);
        }
    }
    else if (method==CV_HOUGH_PROBABILISTIC) {
           for (k=0; k<seq->total; k++) {
               points = (CvPoint*) cvGetSeqElem(seq,k);
               cvLine(arr,points[0],points[1],color,thickness,8,0);
           }
    }
    RETURN
    
}
//---------------------------------SHAPE DESCRIPTOR --------------------------------

void cv_find_contours(int arrRef, int memRef, int mode, int method, int *offsetlist, int offsetlen){
    CvArr *arr;
    CvMemStorage *mem;
    CvSeq *contour = 0;
    CvPoint offset;
    arr = (CvArr*) _get_cvarr_pointer(arrRef);
    if (!arr) {
        MLPutString(stdlink,"Error : bad CvArr reference");
        return;
    }
    mem = (CvMemStorage*) _get_pointer(memRef,CVMEMSTORAGE_REF);
    if (!mem) {
        MLPutString(stdlink,"Error : bad CvMemStorage reference");
        return;
    }
    if (offsetlen!=2) {
        MLPutString(stdlink,"Error : offset must be a two elements List");
        return;
    }
    offset.x = offsetlist[0];
    offset.y = offsetlist[1];
    cvFindContours(arr,mem,&contour,sizeof(CvContour),mode,method,offset);
    MLPutInteger(stdlink,_append_ref((void*)contour,CVSEQ_REF));
}

void cv_get_contour(int seqRef) {
    CvSeq *seq;
    int i;
    long dims[2];
    double *array;
    CvPoint *p;
    GETPOINTER(seq,seqRef,CvSeq*,CVSEQ_REF)
    dims[0] = seq->total;
    dims[1] = 2;
    array = malloc(sizeof(double)*dims[0]*dims[1]);
    for (i=0; i < seq->total; i++) {
        p = CV_GET_SEQ_ELEM(CvPoint,seq,i);
        array[i*2] = p->x;
        array[i*2+1] = p->y;
    }
    MLPutRealArray(stdlink,array,dims,NULL,2);
    free(array);
}

void cv_fit_ellipse2(int arrRef) {
    CvArr *arr;
    CvBox2D box;
    float array[5];
    arr = (CvArr*) _get_cvarr_pointer(arrRef);
    if (!arr) {
        MLPutString(stdlink,"Error : bad CvArr reference");
        return;
    }
    box = cvFitEllipse2(arr);
    array[0] = box.center.x;
    array[1] = box.center.y;
    array[2] = box.size.width;
    array[3] = box.size.height;
    array[4] = box.angle;
    MLPutReal32List(stdlink,array,5);
}

void cv_min_area_rect2(int arrRef) {
    CvArr *arr;
    CvBox2D box;
    float array[5];
    GETCVARRPOINTER(arr,arrRef)
    box = cvMinAreaRect2(arr,NULL);
    array[0] = box.center.x;
    array[1] = box.center.y;
    array[2] = box.size.width;
    array[3] = box.size.height;
    array[4] = box.angle;
    MLPutReal32List(stdlink,array,5);
}

void cv_contour_perimeter(int seqRef) {
    CvSeq *seq;
    GETPOINTER(seq,seqRef,CvSeq*,CVSEQ_REF)
    MLPutReal64(stdlink,cvContourPerimeter(seq));
}

//--------------------------------- READING AND WRITING IMAGES AND VIDEO -------------------------------------
int cv_load_image(char *filename, int iscolor) {
    return _append_ref((void*)cvLoadImage(filename,iscolor),IPLIMAGE_REF);
}

void cv_save_image(char *filename, int imgref){
    CvArr *img;
    GETCVARRPOINTER(img,imgref)
    cvSaveImage(filename,img,NULL);
    RETURN
}

int cv_capture_from_file(char *filename) {
    return _append_ref((void*)cvCaptureFromFile(filename),CVCAPTURE_REF);
}

int cv_capture_from_cam(int index) {
    return _append_ref((void*)cvCaptureFromCAM(index),CVCAPTURE_REF);   
}

void cv_get_frame(int ref, int skip) {
    CvCapture *capture;
    IplImage *frame = 0;
    IplImage *image;
    int f;
    if (skip <= 0) {
        MLPutString(stdlink,"Erreur : skip doit etre > 0");
        return;
    }
    if (capture=(CvCapture*) _get_pointer(ref,CVCAPTURE_REF)) {
        for (f=0; f < skip-1; f++)  cvGrabFrame(capture);
        frame = cvQueryFrame(capture);
        if (frame) {
            image = cvCreateImage(cvSize(frame->width,frame->height),frame->depth,frame->nChannels);
            cvCopy(frame,image,NULL);
            MLPutInteger(stdlink,_append_ref((void*)image,IPLIMAGE_REF));
        }
        else {
            MLPutInteger(stdlink,0);
        }
    }
    else {
        MLPutString(stdlink,"Reference de video incorrecte");
    }
}

void cv_create_video_writer(char *filename,int fourcc, double fps, int width, int height, int is_color) {
    CvVideoWriter *videoWriter;
    videoWriter = cvCreateVideoWriter(filename,fourcc,fps,cvSize(width,height),is_color);
    MLPutInteger(stdlink,_append_ref((void*)videoWriter,CVVIDEOWRITER_REF));
}

void cv_release_video_writer(int writerRef) {
    CvVideoWriter *videoWriter;
    GETPOINTER(videoWriter,writerRef,CvVideoWriter*,CVVIDEOWRITER_REF)
    cvReleaseVideoWriter(&videoWriter);
    tabRef[writerRef]->type = EMPTY_REF;
    RETURN
}

void cv_write_frame(int writerRef, int imgRef) {
    CvVideoWriter *writer;
    IplImage *img;
    GETPOINTER(writer,writerRef,CvVideoWriter*,CVVIDEOWRITER_REF)
    GETPOINTER(img,imgRef,IplImage*,IPLIMAGE_REF)
    cvWriteFrame(writer,img);
    RETURN
}

void cv_video_info(int ref) {
    CvCapture *capture;
    char reponse[301];
    if (capture=(CvCapture*) _get_pointer(ref,CVCAPTURE_REF)) {
#ifdef WIN32
        sprintf_s(reponse,300,"frame rate : %f\nframe count : %f\nwidth : %f\nheight : %f\n",cvGetCaptureProperty(capture,CV_CAP_PROP_FPS),cvGetCaptureProperty(capture,CV_CAP_PROP_FRAME_COUNT),cvGetCaptureProperty(capture,CV_CAP_PROP_FRAME_WIDTH),cvGetCaptureProperty(capture,CV_CAP_PROP_FRAME_HEIGHT));
#else
        sprintf(reponse,"frame rate : %f\nframe count : %f\nwidth : %f\nheight : %f\n",cvGetCaptureProperty(capture,CV_CAP_PROP_FPS),cvGetCaptureProperty(capture,CV_CAP_PROP_FRAME_COUNT),cvGetCaptureProperty(capture,CV_CAP_PROP_FRAME_WIDTH),cvGetCaptureProperty(capture,CV_CAP_PROP_FRAME_HEIGHT));
#endif
        MLPutString(stdlink,reponse);
    }
    else {
        MLPutString(stdlink,"Reference de video incorrecte");
    }
}

void cv_get_capture_property(int captref, int property_id) {
    CvCapture *capture;
    double p;
    GETPOINTER(capture,captref,CvCapture*,CVCAPTURE_REF)
    p = cvGetCaptureProperty(capture,property_id);
    if ((property_id==CV_CAP_PROP_FOURCC)||(property_id==CV_CAP_PROP_FRAME_WIDTH)||(property_id==CV_CAP_PROP_FRAME_HEIGHT)||(property_id==CV_CAP_PROP_FRAME_COUNT)) {
        MLPutInteger(stdlink,(int) p);   
    }
    else {
        MLPutReal64(stdlink,p);
    }
}

void cv_set_capture_property(int captref, int property_id, double value) {
    CvCapture *capture;
    GETPOINTER(capture,captref,CvCapture*,CVCAPTURE_REF)
    cvSetCaptureProperty(capture,property_id,value);
    RETURN
}

void cv_codec_name(int fourcc) {
    char name[5];
    int c;
    c = fourcc;
    name[0] = c & 0xFF;
    name[1] = (c & 0xFF00) >> 8;
    name[2] = (c & 0xFF0000) >> 16;
    name[3] = (c & 0xFF000000) >> 24;
    name[4] = 0;
    MLPutString(stdlink,name);
}

void cv_fourcc(char *name) {
     char c1,c2,c3,c4;
     c1 = name[0];
     c2 = name[1];
     c3 = name[2];
     c4 = name[3];
     if ((c1*c2*c3*c4==0)||(name[4]!=0)) {
         MLPutString(stdlink,"Error : codec name must have 4 characters");
         return;
     }
     MLPutInteger(stdlink,CV_FOURCC(c1,c2,c3,c4));
}
//----------------------- GUI -----------------------------------------------
void cv_named_window(char *name, int flags) {
    cvNamedWindow(name,flags);
    MLPutString(stdlink,"");
}



void cv_show_image(char* name, int ref) {
    if ((ref<0)||(ref>=lastRef)) {
        MLPutString(stdlink,"Error : bad CvArr reference");
        return;
    }
    if (tabRef[ref]->type == IPLIMAGE_REF) {
        cvShowImage(name,(IplImage*) (tabRef[ref]->pointer));
        MLPutString(stdlink,"");
        return;
    }
    if (tabRef[ref]->type == CVMAT_REF) {
        cvShowImage(name,(CvMat*) (tabRef[ref]->pointer));
        MLPutString(stdlink,"");
        return;
    }
    MLPutString(stdlink,"Error : bad CvArr reference");
    return;
}



int show_image(char *filename) {
    IplImage *src;
    src = cvLoadImage(filename,CV_LOAD_IMAGE_COLOR);
    cvNamedWindow(filename,CV_WINDOW_AUTOSIZE);
    cvShowImage(filename,src);
    //cvWaitKey(0);
    cvReleaseImage(&src);
    return 0;
}



void cv_play_video( char* window, int ref, int delay) {
    CvCapture* capture = 0;
    IplImage *frame = 0;
    if (capture=(CvCapture*) _get_pointer(ref,CVCAPTURE_REF)) {
        for (;;) {
            frame = cvQueryFrame(capture);
            if (!frame) break;
            //cvNamedWindow(window,CV_WINDOW_AUTOSIZE);
            cvShowImage(window,frame);
            if (cvWaitKey(delay)>0) break;
        }
        MLPutString(stdlink,"");
    }
    else {
        MLPutString(stdlink,"Reference de video incorrecte");
    }
}
//----------------------------- EXTRA GUI FUNCTIONS ------------------------
void _mouse_event_callback(int event, int x, int y, int flags, void *eventType) {
    if (event==*((int*)eventType)) {
        lastMousePoint.x = x;
        lastMousePoint.y = y;
        mouseEventDone = 1;
    }
}

void cv_wait_mouse_event(char* windowName, int eventType) {
    int p[2];
    int key=0;
    lastMousePoint.x = -1; lastMousePoint.y = -1;
    cvSetMouseCallback(windowName,_mouse_event_callback,&eventType);
    mouseEventDone = 0;
    while ((!mouseEventDone)&&(key!=27)) key = cvWaitKey(20);
    p[0] = lastMousePoint.x; p[1] = lastMousePoint.y;
    MLPutIntegerList(stdlink,p,2);
}

#define CROSS_WIDTH 7
#define CROSS_LINE_WIDTH 1
void _point_selected_callback(int event, int x, int y, int flags, void *param) {
    CvPoint *p;
    imagePoints *imgpts;
    int *c;
    imgpts = (imagePoints*) param;
    c = imgpts->color;
    if (event==CV_EVENT_LBUTTONDOWN) {
        p = malloc(sizeof(CvPoint));
        p->x = x;
        p->y = y;
        cvSeqPush(imgpts->points,p);
        cvLine(imgpts->image,cvPoint(p->x-CROSS_WIDTH,p->y),cvPoint(p->x+CROSS_WIDTH,p->y),CV_RGB(c[0],c[1],c[2]),CROSS_LINE_WIDTH,8,0);
        cvLine(imgpts->image,cvPoint(p->x,p->y-CROSS_WIDTH),cvPoint(p->x,p->y+CROSS_WIDTH),CV_RGB(c[0],c[1],c[2]),CROSS_LINE_WIDTH,8,0);
        cvShowImage(imgpts->window,imgpts->image);
    }
}

void cv_get_selected_points(char *windowName, int imgRef, int *color, int colorlen) {
    IplImage *img;
    IplImage *display;
    CvSeq *points;
    CvMemStorage *storage;
    int k;
    int key = 0;
    int *pointsList;
    long dims[2];
    CvPoint *p;
    imagePoints imgpts;
    img = (IplImage*) _get_pointer(imgRef,IPLIMAGE_REF);
    if (!img) {
        MLPutString(stdlink,"Error : bad IplImage reference");
        return;
    }
    if (colorlen!=3) {
        MLPutString(stdlink,"Error : color must be a list of 3 integers");
        return;
    }
    display = cvCloneImage(img);
    storage = cvCreateMemStorage(0);
    points = cvCreateSeq(0,sizeof(CvSeq),sizeof(CvPoint),storage);
    cvNamedWindow(windowName,CV_WINDOW_AUTOSIZE);
    imgpts.image = display; imgpts.points = points; imgpts.window = windowName; imgpts.color = color;
    cvSetMouseCallback(windowName,_point_selected_callback,&imgpts);
    cvShowImage(windowName,display);
    while (key!=27) key = cvWaitKey(20);
    cvSetMouseCallback(windowName,NULL,NULL);
    pointsList = malloc(sizeof(int)*points->total*2);
    for (k=0; k<points->total;k++) {
        p = (CvPoint*) cvGetSeqElem(points,k);
        pointsList[2*k] = p->x;
        pointsList[2*k+1] = p->y;
    }
    dims[0] = points->total;
    dims[1] = 2;
    MLPutIntegerArray(stdlink,pointsList,dims,NULL,2);
    free(pointsList);
    cvReleaseMemStorage(&storage);
}

#if MACINTOSH_MATHLINK

int main( int argc, char* argv[])
{
	/* Due to a bug in some standard C libraries that have shipped with
	 * MPW, zero is passed to MLMain below.  (If you build this program
	 * as an MPW tool, you can change the zero to argc.)
	 */
	argc = argc; /* suppress warning */
	return MLMain( 0, argv);
}

#elif WINDOWS_MATHLINK

#if __BORLANDC__
#pragma argsused
#endif

int PASCAL WinMain( HINSTANCE hinstCurrent, HINSTANCE hinstPrevious, LPSTR lpszCmdLine, int nCmdShow)
{
	char  buff[512];
	char FAR * buff_start = buff;
	char FAR * argv[32];
	char FAR * FAR * argv_end = argv + 32;

	hinstPrevious = hinstPrevious; /* suppress warning */

	if( !MLInitializeIcon( hinstCurrent, nCmdShow)) return 1;
	MLScanString( argv, &argv_end, &lpszCmdLine, &buff_start);
	return MLMain( argv_end - argv, argv);
}

#else

int main(int argc, char* argv[])
{
	return MLMain(argc, argv);
}

#endif
